@@ -24,8 +24,6 @@
 
 #include "ngx-queue.h"
 
-#include "ev.h"
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -43,23 +41,6 @@
 #include <pthread.h>
 #include <signal.h>
 
-struct uv__io_s;
-struct uv_loop_s;
-
-typedef struct uv__io_s uv__io_t;
-typedef void (*uv__io_cb)(struct uv_loop_s* loop, uv__io_t* handle, int events);
-
-struct uv__io_s {
-  ev_io io_watcher;
-};
-
-struct uv__work {
-  void (*work)(struct uv__work *w);
-  void (*done)(struct uv__work *w);
-  struct uv_loop_s* loop;
-  ngx_queue_t wq;
-};
-
 #if defined(__linux__)
 # include "uv-linux.h"
 #elif defined(__sun)
@@ -73,6 +54,49 @@ struct uv__work {
 # include "uv-bsd.h"
 #endif
 
+#ifndef UV_IO_PRIVATE_PLATFORM_FIELDS
+# define UV_IO_PRIVATE_PLATFORM_FIELDS /* empty */
+#endif
+
+#define UV_IO_PRIVATE_FIELDS                                                  \
+  UV_IO_PRIVATE_PLATFORM_FIELDS                                               \
+
+struct uv__io_s;
+struct uv__async;
+struct uv_loop_s;
+
+typedef void (*uv__io_cb)(struct uv_loop_s* loop,
+                          struct uv__io_s* w,
+                          unsigned int events);
+typedef struct uv__io_s uv__io_t;
+
+struct uv__io_s {
+  uv__io_cb cb;
+  ngx_queue_t pending_queue;
+  ngx_queue_t watcher_queue;
+  unsigned int pevents; /* Pending event mask i.e. mask at next tick. */
+  unsigned int events;  /* Current event mask. */
+  int fd;
+  UV_IO_PRIVATE_FIELDS
+};
+
+typedef void (*uv__async_cb)(struct uv_loop_s* loop,
+                             struct uv__async* w,
+                             unsigned int nevents);
+
+struct uv__async {
+  uv__async_cb cb;
+  uv__io_t io_watcher;
+  int wfd;
+};
+
+struct uv__work {
+  void (*work)(struct uv__work *w);
+  void (*done)(struct uv__work *w, int status);
+  struct uv_loop_s* loop;
+  ngx_queue_t wq;
+};
+
 #ifndef UV_PLATFORM_SEM_T
 # define UV_PLATFORM_SEM_T sem_t
 #endif
@@ -106,6 +130,24 @@ typedef pthread_t uv_thread_t;
 typedef pthread_mutex_t uv_mutex_t;
 typedef pthread_rwlock_t uv_rwlock_t;
 typedef UV_PLATFORM_SEM_T uv_sem_t;
+typedef pthread_cond_t uv_cond_t;
+
+
+#if defined(__APPLE__) && defined(__MACH__)
+
+typedef struct {
+  unsigned int n;
+  unsigned int count;
+  uv_mutex_t mutex;
+  uv_sem_t turnstile1;
+  uv_sem_t turnstile2;
+} uv_barrier_t;
+
+#else /* defined(__APPLE__) && defined(__MACH__) */
+
+typedef pthread_barrier_t uv_barrier_t;
+
+#endif /* defined(__APPLE__) && defined(__MACH__) */
 
 /* Platform-specific definitions for uv_spawn support. */
 typedef gid_t uv_gid_t;
@@ -121,7 +163,12 @@ typedef struct {
 
 #define UV_LOOP_PRIVATE_FIELDS                                                \
   unsigned long flags;                                                        \
-  struct ev_loop* ev;                                                         \
+  int backend_fd;                                                             \
+  ngx_queue_t pending_queue;                                                  \
+  ngx_queue_t watcher_queue;                                                  \
+  uv__io_t** watchers;                                                        \
+  unsigned int nwatchers;                                                     \
+  unsigned int nfds;                                                          \
   ngx_queue_t wq;                                                             \
   uv_mutex_t wq_mutex;                                                        \
   uv_async_t wq_async;                                                        \
@@ -131,16 +178,17 @@ typedef struct {
   ngx_queue_t check_handles;                                                  \
   ngx_queue_t idle_handles;                                                   \
   ngx_queue_t async_handles;                                                  \
-  uv__io_t async_watcher;                                                     \
-  int async_pipefd[2];                                                        \
+  struct uv__async async_watcher;                                             \
   /* RB_HEAD(uv__timers, uv_timer_s) */                                       \
   struct uv__timers {                                                         \
     struct uv_timer_s* rbh_root;                                              \
   } timer_handles;                                                            \
   uint64_t time;                                                              \
-  void* signal_ctx;                                                           \
+  int signal_pipefd[2];                                                       \
+  uv__io_t signal_io_watcher;                                                 \
   uv_signal_t child_watcher;                                                  \
   int emfile_fd;                                                              \
+  uint64_t timer_counter;                                                     \
   UV_PLATFORM_LOOP_FIELDS                                                     \
 
 #define UV_REQ_TYPE_PRIVATE /* empty */
@@ -178,24 +226,20 @@ typedef struct {
 #define UV_STREAM_PRIVATE_FIELDS                                              \
   uv_connect_t *connect_req;                                                  \
   uv_shutdown_t *shutdown_req;                                                \
-  uv__io_t read_watcher;                                                      \
-  uv__io_t write_watcher;                                                     \
+  uv__io_t io_watcher;                                                        \
   ngx_queue_t write_queue;                                                    \
   ngx_queue_t write_completed_queue;                                          \
   uv_connection_cb connection_cb;                                             \
   int delayed_error;                                                          \
   int accepted_fd;                                                            \
-  int fd;                                                                     \
   UV_STREAM_PRIVATE_PLATFORM_FIELDS                                           \
 
 #define UV_TCP_PRIVATE_FIELDS /* empty */
 
 #define UV_UDP_PRIVATE_FIELDS                                                 \
-  int fd;                                                                     \
   uv_alloc_cb alloc_cb;                                                       \
   uv_udp_recv_cb recv_cb;                                                     \
-  uv__io_t read_watcher;                                                      \
-  uv__io_t write_watcher;                                                     \
+  uv__io_t io_watcher;                                                        \
   ngx_queue_t write_queue;                                                    \
   ngx_queue_t write_completed_queue;                                          \
 
@@ -203,7 +247,6 @@ typedef struct {
   const char* pipe_fname; /* strdup'ed */
 
 #define UV_POLL_PRIVATE_FIELDS                                                \
-  int fd;                                                                     \
   uv__io_t io_watcher;
 
 #define UV_PREPARE_PRIVATE_FIELDS                                             \
@@ -219,12 +262,12 @@ typedef struct {
   ngx_queue_t queue;
 
 #define UV_ASYNC_PRIVATE_FIELDS                                               \
-  volatile sig_atomic_t pending;                                              \
   uv_async_cb async_cb;                                                       \
-  ngx_queue_t queue;
+  ngx_queue_t queue;                                                          \
+  int pending;                                                                \
 
 #define UV_TIMER_PRIVATE_FIELDS                                               \
-  /* RB_ENTRY(uv_timer_s) node; */                                            \
+  /* RB_ENTRY(uv_timer_s) tree_entry; */                                      \
   struct {                                                                    \
     struct uv_timer_s* rbe_left;                                              \
     struct uv_timer_s* rbe_right;                                             \
@@ -233,7 +276,8 @@ typedef struct {
   } tree_entry;                                                               \
   uv_timer_cb timer_cb;                                                       \
   uint64_t timeout;                                                           \
-  uint64_t repeat;
+  uint64_t repeat;                                                            \
+  uint64_t start_id;
 
 #define UV_GETADDRINFO_PRIVATE_FIELDS                                         \
   struct uv__work work_req;                                                   \
@@ -261,7 +305,6 @@ typedef struct {
   double atime;                                                               \
   double mtime;                                                               \
   struct uv__work work_req;                                                   \
-  struct stat statbuf;                                                        \
 
 #define UV_WORK_PRIVATE_FIELDS                                                \
   struct uv__work work_req;
@@ -271,7 +314,16 @@ typedef struct {
   int mode;
 
 #define UV_SIGNAL_PRIVATE_FIELDS                                              \
-  ngx_queue_t queue;
+  /* RB_ENTRY(uv_signal_s) tree_entry; */                                     \
+  struct {                                                                    \
+    struct uv_signal_s* rbe_left;                                             \
+    struct uv_signal_s* rbe_right;                                            \
+    struct uv_signal_s* rbe_parent;                                           \
+    int rbe_color;                                                            \
+  } tree_entry;                                                               \
+  /* Use two counters here so we don have to fiddle with atomics. */          \
+  unsigned int caught_signals;                                                \
+  unsigned int dispatched_signals;
 
 #define UV_FS_EVENT_PRIVATE_FIELDS                                            \
   uv_fs_event_cb cb;                                                          \
