@@ -39,9 +39,6 @@
 # include <sys/time.h>
 # include <sys/select.h>
 
-/* ev.h is overwriting EV_ERROR from sys/event.h */
-#define EV_ERROR_ORIG 0x4000
-
 /* Forward declaration */
 typedef struct uv__stream_select_s uv__stream_select_t;
 
@@ -53,13 +50,15 @@ struct uv__stream_select_s {
   uv_async_t async;
   int events;
   int fake_fd;
+  int int_fd;
+  int fd;
 };
 #endif /* defined(__APPLE__) */
 
 static void uv__stream_connect(uv_stream_t*);
 static void uv__write(uv_stream_t* stream);
 static void uv__read(uv_stream_t* stream);
-static void uv__stream_io(uv_loop_t* loop, uv__io_t* w, int events);
+static void uv__stream_io(uv_loop_t* loop, uv__io_t* w, unsigned int events);
 
 
 /* Used by the accept() EMFILE party trick. */
@@ -109,7 +108,6 @@ void uv__stream_init(uv_loop_t* loop,
   stream->connect_req = NULL;
   stream->shutdown_req = NULL;
   stream->accepted_fd = -1;
-  stream->fd = -1;
   stream->delayed_error = 0;
   ngx_queue_init(&stream->write_queue);
   ngx_queue_init(&stream->write_completed_queue);
@@ -122,69 +120,124 @@ void uv__stream_init(uv_loop_t* loop,
   stream->select = NULL;
 #endif /* defined(__APPLE_) */
 
-  uv__io_init(&stream->read_watcher, uv__stream_io, -1, 0);
-  uv__io_init(&stream->write_watcher, uv__stream_io, -1, 0);
+  uv__io_init(&stream->io_watcher, uv__stream_io, -1);
 }
 
 
 #if defined(__APPLE__)
-void uv__stream_osx_select(void* arg) {
+static void uv__stream_osx_select(void* arg) {
   uv_stream_t* stream;
   uv__stream_select_t* s;
-  fd_set read;
-  fd_set write;
-  fd_set error;
-  struct timeval timeout;
+  char buf[1024];
+  fd_set sread;
+  fd_set swrite;
+  fd_set serror;
   int events;
   int fd;
   int r;
+  int max_fd;
 
   stream = arg;
   s = stream->select;
-  fd = stream->fd;
+  fd = stream->io_watcher.fd;
+
+  if (fd > s->int_fd)
+    max_fd = fd;
+  else
+    max_fd = s->int_fd;
 
   while (1) {
     /* Terminate on semaphore */
-    if (uv_sem_trywait(&s->sem) == 0) break;
+    if (uv_sem_trywait(&s->sem) == 0)
+      break;
 
     /* Watch fd using select(2) */
-    FD_ZERO(&read);
-    FD_ZERO(&write);
-    FD_ZERO(&error);
-    FD_SET(fd, &read);
-    FD_SET(fd, &write);
-    FD_SET(fd, &error);
+    FD_ZERO(&sread);
+    FD_ZERO(&swrite);
+    FD_ZERO(&serror);
 
-    timeout.tv_sec = 0;
-    timeout.tv_usec = 250000; /* 250 ms timeout */
-    r = select(fd + 1, &read, &write, &error, &timeout);
+    if (uv_is_readable(stream))
+      FD_SET(fd, &sread);
+    if (uv_is_writable(stream))
+      FD_SET(fd, &swrite);
+    FD_SET(fd, &serror);
+    FD_SET(s->int_fd, &sread);
+
+    /* Wait indefinitely for fd events */
+    r = select(max_fd + 1, &sread, &swrite, &serror, NULL);
     if (r == -1) {
-      if (errno == EINTR) continue;
+      if (errno == EINTR)
+        continue;
+
       /* XXX: Possible?! */
       abort();
     }
 
     /* Ignore timeouts */
-    if (r == 0) continue;
+    if (r == 0)
+      continue;
+
+    /* Empty socketpair's buffer in case of interruption */
+    if (FD_ISSET(s->int_fd, &sread))
+      while (1) {
+        r = read(s->int_fd, buf, sizeof(buf));
+
+        if (r == sizeof(buf))
+          continue;
+
+        if (r != -1)
+          break;
+
+        if (errno == EAGAIN || errno == EWOULDBLOCK)
+          break;
+
+        if (errno == EINTR)
+          continue;
+
+        abort();
+      }
 
     /* Handle events */
     events = 0;
-    if (FD_ISSET(fd, &read)) events |= UV__IO_READ;
-    if (FD_ISSET(fd, &write)) events |= UV__IO_WRITE;
-    if (FD_ISSET(fd, &error)) events |= UV__IO_ERROR;
+    if (FD_ISSET(fd, &sread))
+      events |= UV__POLLIN;
+    if (FD_ISSET(fd, &swrite))
+      events |= UV__POLLOUT;
+    if (FD_ISSET(fd, &serror))
+      events |= UV__POLLERR;
 
     uv_mutex_lock(&s->mutex);
     s->events |= events;
     uv_mutex_unlock(&s->mutex);
 
-    if (events != 0) uv_async_send(&s->async);
+    if (events != 0)
+      uv_async_send(&s->async);
   }
 }
 
 
-void uv__stream_osx_select_cb(uv_async_t* handle, int status) {
-  uv_stream_t* stream;
+static void uv__stream_osx_interrupt_select(uv_stream_t* stream) {
+  /* Notify select() thread about state change */
   uv__stream_select_t* s;
+  int r;
+
+  s = stream->select;
+
+  /* Interrupt select() loop
+   * NOTE: fake_fd and int_fd are socketpair(), thus writing to one will
+   * emit read event on other side
+   */
+  do
+    r = write(s->fake_fd, "x", 1);
+  while (r == -1 && errno == EINTR);
+
+  assert(r == 1);
+}
+
+
+static void uv__stream_osx_select_cb(uv_async_t* handle, int status) {
+  uv__stream_select_t* s;
+  uv_stream_t* stream;
   int events;
 
   s = container_of(handle, uv__stream_select_t, async);
@@ -196,142 +249,127 @@ void uv__stream_osx_select_cb(uv_async_t* handle, int status) {
   s->events = 0;
   uv_mutex_unlock(&s->mutex);
 
+  assert(0 == (events & UV__POLLERR));
+
   /* Invoke callback on event-loop */
-  if ((events & UV__IO_READ) && uv__io_active(&stream->read_watcher)) {
-    uv__stream_io(stream->loop, &stream->read_watcher, UV__IO_READ);
-  }
-  if ((events & UV__IO_WRITE) && uv__io_active(&stream->write_watcher)) {
-    uv__stream_io(stream->loop, &stream->write_watcher, UV__IO_WRITE);
-  }
-  if (events & UV__IO_ERROR) {
-    /* XXX: Handle it! */
-    uv__stream_io(stream->loop, NULL, UV__IO_ERROR);
-  }
+  if ((events & UV__POLLIN) && uv__io_active(&stream->io_watcher, UV__POLLIN))
+    uv__stream_io(stream->loop, &stream->io_watcher, UV__POLLIN);
+
+  if ((events & UV__POLLOUT) && uv__io_active(&stream->io_watcher, UV__POLLOUT))
+    uv__stream_io(stream->loop, &stream->io_watcher, UV__POLLOUT);
 }
 
 
-void uv__stream_osx_cb_close(uv_handle_t* async) {
-  /* Free container */
-  free(container_of(async, uv__stream_select_t, async));
+static void uv__stream_osx_cb_close(uv_handle_t* async) {
+  uv__stream_select_t* s;
+
+  s = container_of(async, uv__stream_select_t, async);
+  free(s);
 }
 
 
-int uv__stream_try_select(uv_stream_t* stream, int fd) {
+int uv__stream_try_select(uv_stream_t* stream, int* fd) {
   /*
    * kqueue doesn't work with some files from /dev mount on osx.
    * select(2) in separate thread for those fds
    */
 
-  int kq;
-  int ret;
   struct kevent filter[1];
   struct kevent events[1];
   struct timespec timeout;
   uv__stream_select_t* s;
+  int fds[2];
+  int ret;
+  int kq;
 
   kq = kqueue();
-  if (kq < 0) {
+  if (kq == -1) {
     fprintf(stderr, "(libuv) Failed to create kqueue (%d)\n", errno);
-    abort();
+    return uv__set_sys_error(stream->loop, errno);
   }
 
-  EV_SET(&filter[0], fd, EVFILT_READ, EV_ADD | EV_ENABLE, 0, 0, 0);
+  EV_SET(&filter[0], *fd, EVFILT_READ, EV_ADD | EV_ENABLE, 0, 0, 0);
 
   /* Use small timeout, because we only want to capture EINVALs */
   timeout.tv_sec = 0;
   timeout.tv_nsec = 1;
 
   ret = kevent(kq, filter, 1, events, 1, &timeout);
-  close(kq);
-  if (ret < 1) return -1;
-  if ((events[0].flags & EV_ERROR_ORIG) == 0 || events[0].data != EINVAL) {
-    return -1;
-  }
+  SAVE_ERRNO(close(kq));
+
+  if (ret == -1)
+    return uv__set_sys_error(stream->loop, errno);
+
+  if (ret == 0 || (events[0].flags & EV_ERROR) == 0 || events[0].data != EINVAL)
+    return 0;
 
   /* At this point we definitely know that this fd won't work with kqueue */
   s = malloc(sizeof(*s));
-  if (s == NULL) {
-    /* TODO: Return error */
-    abort();
+  if (s == NULL)
+    return uv__set_artificial_error(stream->loop, UV_ENOMEM);
+
+  s->fd = *fd;
+
+  if (uv_async_init(stream->loop, &s->async, uv__stream_osx_select_cb)) {
+    SAVE_ERRNO(free(s));
+    return uv__set_sys_error(stream->loop, errno);
   }
 
-  if (uv_async_init(stream->loop,
-                    &s->async,
-                    uv__stream_osx_select_cb)) {
-    return -1;
-  }
   s->async.flags |= UV__HANDLE_INTERNAL;
-  uv__handle_unref((uv_handle_t*) &s->async);
+  uv__handle_unref(&s->async);
 
-  if (uv_sem_init(&s->sem, 0)) goto fatal1;
-  if (uv_mutex_init(&s->mutex)) goto fatal2;
+  if (uv_sem_init(&s->sem, 0))
+    goto fatal1;
 
-  /* Create fake fd for io watcher */
-  s->fake_fd = socket(AF_UNIX, SOCK_STREAM, 0);
-  if (s->fake_fd == -1) goto fatal3;
+  if (uv_mutex_init(&s->mutex))
+    goto fatal2;
 
-  if (uv_thread_create(&s->thread, uv__stream_osx_select, stream)) {
+  /* Create fds for io watcher and to interrupt the select() loop. */
+  if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds))
+    goto fatal3;
+
+  s->fake_fd = fds[0];
+  s->int_fd = fds[1];
+
+  if (uv_thread_create(&s->thread, uv__stream_osx_select, stream))
     goto fatal4;
-  }
 
   s->stream = stream;
   stream->select = s;
+  *fd = s->fake_fd;
 
   return 0;
 
 fatal4:
   close(s->fake_fd);
+  close(s->int_fd);
+  s->fake_fd = -1;
+  s->int_fd = -1;
 fatal3:
   uv_mutex_destroy(&s->mutex);
 fatal2:
   uv_sem_destroy(&s->sem);
 fatal1:
   uv_close((uv_handle_t*) &s->async, uv__stream_osx_cb_close);
-
-  free(s);
-  return -1;
+  return uv__set_sys_error(stream->loop, errno);
 }
 #endif /* defined(__APPLE__) */
 
 
 int uv__stream_open(uv_stream_t* stream, int fd, int flags) {
-  socklen_t yes;
-
   assert(fd >= 0);
-  stream->fd = fd;
-
   stream->flags |= flags;
 
   if (stream->type == UV_TCP) {
-    /* Reuse the port address if applicable. */
-    yes = 1;
-    if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof yes) == -1) {
-      uv__set_sys_error(stream->loop, errno);
-      return -1;
-    }
-
-    if ((stream->flags & UV_TCP_NODELAY) &&
-        uv__tcp_nodelay((uv_tcp_t*)stream, 1)) {
-      return -1;
-    }
+    if ((stream->flags & UV_TCP_NODELAY) && uv__tcp_nodelay(fd, 1))
+      return uv__set_sys_error(stream->loop, errno);
 
     /* TODO Use delay the user passed in. */
-    if ((stream->flags & UV_TCP_KEEPALIVE) &&
-        uv__tcp_keepalive((uv_tcp_t*)stream, 1, 60)) {
-      return -1;
-    }
+    if ((stream->flags & UV_TCP_KEEPALIVE) && uv__tcp_keepalive(fd, 1, 60))
+      return uv__set_sys_error(stream->loop, errno);
   }
 
-#if defined(__APPLE__)
-  if (uv__stream_try_select(stream, fd) == 0) {
-    /* Use fake fd */
-    fd = ((uv__stream_select_t*) stream->select)->fake_fd;
-  }
-#endif /* defined(__APPLE__) */
-
-  /* Associate the fd with each watcher. */
-  uv__io_set(&stream->read_watcher, uv__stream_io, fd, UV__IO_READ);
-  uv__io_set(&stream->write_watcher, uv__stream_io, fd, UV__IO_WRITE);
+  stream->io_watcher.fd = fd;
 
   return 0;
 }
@@ -341,6 +379,7 @@ void uv__stream_destroy(uv_stream_t* stream) {
   uv_write_t* req;
   ngx_queue_t* q;
 
+  assert(!uv__io_active(&stream->io_watcher, UV__POLLIN | UV__POLLOUT));
   assert(stream->flags & UV_CLOSED);
 
   if (stream->connect_req) {
@@ -431,27 +470,37 @@ static int uv__emfile_trick(uv_loop_t* loop, int accept_fd) {
 }
 
 
-void uv__server_io(uv_loop_t* loop, uv__io_t* w, int events) {
+#if defined(UV_HAVE_KQUEUE)
+# define UV_DEC_BACKLOG(w) w->rcount--;
+#else
+# define UV_DEC_BACKLOG(w) /* no-op */
+#endif /* defined(UV_HAVE_KQUEUE) */
+
+
+void uv__server_io(uv_loop_t* loop, uv__io_t* w, unsigned int events) {
   static int use_emfile_trick = -1;
   uv_stream_t* stream;
   int fd;
   int r;
 
-  stream = container_of(w, uv_stream_t, read_watcher);
-  assert(events == UV__IO_READ);
+  stream = container_of(w, uv_stream_t, io_watcher);
+  assert(events == UV__POLLIN);
+  assert(stream->accepted_fd == -1);
   assert(!(stream->flags & UV_CLOSING));
 
-  if (stream->accepted_fd >= 0) {
-    uv__io_stop(loop, &stream->read_watcher);
-    return;
-  }
+  if (stream->accepted_fd == -1)
+    uv__io_start(stream->loop, &stream->io_watcher, UV__POLLIN);
 
   /* connection_cb can close the server socket while we're
    * in the loop so check it on each iteration.
    */
-  while (stream->fd != -1) {
-    assert(stream->accepted_fd < 0);
-    fd = uv__accept(stream->fd);
+  while (uv__stream_fd(stream) != -1) {
+    assert(stream->accepted_fd == -1);
+#if defined(UV_HAVE_KQUEUE)
+    if (w->rcount <= 0)
+      return;
+#endif /* defined(UV_HAVE_KQUEUE) */
+    fd = uv__accept(uv__stream_fd(stream));
 
     if (fd == -1) {
       switch (errno) {
@@ -462,6 +511,7 @@ void uv__server_io(uv_loop_t* loop, uv__io_t* w, int events) {
         return; /* Not an error. */
 
       case ECONNABORTED:
+        UV_DEC_BACKLOG(w)
         continue; /* Ignore. */
 
       case EMFILE:
@@ -472,9 +522,11 @@ void uv__server_io(uv_loop_t* loop, uv__io_t* w, int events) {
         }
 
         if (use_emfile_trick) {
-          SAVE_ERRNO(r = uv__emfile_trick(loop, stream->fd));
-          if (r == 0)
+          SAVE_ERRNO(r = uv__emfile_trick(loop, uv__stream_fd(stream)));
+          if (r == 0) {
+            UV_DEC_BACKLOG(w)
             continue;
+          }
         }
 
         /* Fall through. */
@@ -486,12 +538,14 @@ void uv__server_io(uv_loop_t* loop, uv__io_t* w, int events) {
       }
     }
 
+    UV_DEC_BACKLOG(w)
+
     stream->accepted_fd = fd;
     stream->connection_cb(stream, 0);
 
     if (stream->accepted_fd != -1) {
       /* The user hasn't yet accepted called uv_accept() */
-      uv__io_stop(loop, &stream->read_watcher);
+      uv__io_stop(loop, &stream->io_watcher, UV__POLLIN);
       return;
     }
 
@@ -504,6 +558,9 @@ void uv__server_io(uv_loop_t* loop, uv__io_t* w, int events) {
 }
 
 
+#undef UV_DEC_BACKLOG
+
+
 int uv_accept(uv_stream_t* server, uv_stream_t* client) {
   uv_stream_t* streamServer;
   uv_stream_t* streamClient;
@@ -524,15 +581,31 @@ int uv_accept(uv_stream_t* server, uv_stream_t* client) {
     goto out;
   }
 
-  if (uv__stream_open(streamClient, streamServer->accepted_fd,
-        UV_STREAM_READABLE | UV_STREAM_WRITABLE)) {
-    /* TODO handle error */
-    close(streamServer->accepted_fd);
-    streamServer->accepted_fd = -1;
-    goto out;
+  switch (streamClient->type) {
+    case UV_NAMED_PIPE:
+    case UV_TCP:
+      if (uv__stream_open(streamClient, streamServer->accepted_fd,
+            UV_STREAM_READABLE | UV_STREAM_WRITABLE)) {
+        /* TODO handle error */
+        close(streamServer->accepted_fd);
+        streamServer->accepted_fd = -1;
+        goto out;
+      }
+      break;
+
+    case UV_UDP:
+      if (uv_udp_open((uv_udp_t*) client, streamServer->accepted_fd)) {
+        close(streamServer->accepted_fd);
+        streamServer->accepted_fd = -1;
+        goto out;
+      }
+      break;
+
+    default:
+      assert(0);
   }
 
-  uv__io_start(streamServer->loop, &streamServer->read_watcher);
+  uv__io_start(streamServer->loop, &streamServer->io_watcher, UV__POLLIN);
   streamServer->accepted_fd = -1;
   status = 0;
 
@@ -566,33 +639,13 @@ int uv_listen(uv_stream_t* stream, int backlog, uv_connection_cb cb) {
 }
 
 
-uv_write_t* uv_write_queue_head(uv_stream_t* stream) {
-  ngx_queue_t* q;
-  uv_write_t* req;
-
-  if (ngx_queue_empty(&stream->write_queue)) {
-    return NULL;
-  }
-
-  q = ngx_queue_head(&stream->write_queue);
-  if (!q) {
-    return NULL;
-  }
-
-  req = ngx_queue_data(q, struct uv_write_s, queue);
-  assert(req);
-
-  return req;
-}
-
-
 static void uv__drain(uv_stream_t* stream) {
   uv_shutdown_t* req;
 
-  assert(!uv_write_queue_head(stream));
+  assert(ngx_queue_empty(&stream->write_queue));
   assert(stream->write_queue_size == 0);
 
-  uv__io_stop(stream->loop, &stream->write_watcher);
+  uv__io_stop(stream->loop, &stream->io_watcher, UV__POLLOUT);
 
   /* Shutdown? */
   if ((stream->flags & UV_STREAM_SHUTTING) &&
@@ -604,7 +657,7 @@ static void uv__drain(uv_stream_t* stream) {
     stream->shutdown_req = NULL;
     uv__req_unregister(stream->loop, req);
 
-    if (shutdown(stream->fd, SHUT_WR)) {
+    if (shutdown(uv__stream_fd(stream), SHUT_WR)) {
       /* Error. Report it. User should call uv_close(). */
       uv__set_sys_error(stream->loop, errno);
       if (req->cb) {
@@ -646,37 +699,43 @@ static void uv__write_req_finish(uv_write_t* req) {
    * callback called in the near future.
    */
   ngx_queue_insert_tail(&stream->write_completed_queue, &req->queue);
-  uv__io_feed(stream->loop, &stream->write_watcher, UV__IO_WRITE);
+  uv__io_feed(stream->loop, &stream->io_watcher);
+}
+
+
+static int uv__handle_fd(uv_handle_t* handle) {
+  switch (handle->type) {
+    case UV_NAMED_PIPE:
+    case UV_TCP:
+      return ((uv_stream_t*) handle)->io_watcher.fd;
+
+    case UV_UDP:
+      return ((uv_udp_t*) handle)->io_watcher.fd;
+
+    default:
+      return -1;
+  }
 }
 
 
-/* On success returns NULL. On error returns a pointer to the write request
- * which had the error.
- */
 static void uv__write(uv_stream_t* stream) {
-  uv_write_t* req;
   struct iovec* iov;
+  ngx_queue_t* q;
+  uv_write_t* req;
   int iovcnt;
   ssize_t n;
 
-  if (stream->flags & UV_CLOSING) {
-    /* Handle was closed this tick. We've received a stale
-     * 'is writable' callback from the event loop, ignore.
-     */
-    return;
-  }
-
 start:
 
-  assert(stream->fd >= 0);
+  assert(uv__stream_fd(stream) >= 0);
 
-  /* Get the request at the head of the queue. */
-  req = uv_write_queue_head(stream);
-  if (!req) {
+  if (ngx_queue_empty(&stream->write_queue)) {
     assert(stream->write_queue_size == 0);
     return;
   }
 
+  q = ngx_queue_head(&stream->write_queue);
+  req = ngx_queue_data(q, uv_write_t, queue);
   assert(req->handle == stream);
 
   /*
@@ -696,7 +755,7 @@ start:
     struct msghdr msg;
     char scratch[64];
     struct cmsghdr *cmsg;
-    int fd_to_send = req->send_handle->fd;
+    int fd_to_send = uv__handle_fd((uv_handle_t*) req->send_handle);
 
     assert(fd_to_send >= 0);
 
@@ -722,15 +781,15 @@ start:
     }
 
     do {
-      n = sendmsg(stream->fd, &msg, 0);
+      n = sendmsg(uv__stream_fd(stream), &msg, 0);
     }
     while (n == -1 && errno == EINTR);
   } else {
     do {
       if (iovcnt == 1) {
-        n = write(stream->fd, iov[0].iov_base, iov[0].iov_len);
+        n = write(uv__stream_fd(stream), iov[0].iov_base, iov[0].iov_len);
       } else {
-        n = writev(stream->fd, iov, iovcnt);
+        n = writev(uv__stream_fd(stream), iov, iovcnt);
       }
     }
     while (n == -1 && errno == EINTR);
@@ -802,7 +861,7 @@ start:
   assert(!(stream->flags & UV_STREAM_BLOCKING));
 
   /* We're not done. */
-  uv__io_start(stream->loop, &stream->write_watcher);
+  uv__io_start(stream->loop, &stream->io_watcher, UV__POLLOUT);
 }
 
 
@@ -827,15 +886,15 @@ static void uv__write_callbacks(uv_stream_t* stream) {
   assert(ngx_queue_empty(&stream->write_completed_queue));
 
   /* Write queue drained. */
-  if (!uv_write_queue_head(stream)) {
+  if (ngx_queue_empty(&stream->write_queue))
     uv__drain(stream);
-  }
 }
 
 
 static uv_handle_type uv__handle_type(int fd) {
   struct sockaddr_storage ss;
   socklen_t len;
+  int type;
 
   memset(&ss, 0, sizeof(ss));
   len = sizeof(ss);
@@ -843,14 +902,25 @@ static uv_handle_type uv__handle_type(int fd) {
   if (getsockname(fd, (struct sockaddr*)&ss, &len))
     return UV_UNKNOWN_HANDLE;
 
-  switch (ss.ss_family) {
-  case AF_UNIX:
-    return UV_NAMED_PIPE;
-  case AF_INET:
-  case AF_INET6:
-    return UV_TCP;
+  len = sizeof type;
+
+  if (getsockopt(fd, SOL_SOCKET, SO_TYPE, &type, &len))
+    return UV_UNKNOWN_HANDLE;
+
+  if (type == SOCK_STREAM) {
+    switch (ss.ss_family) {
+      case AF_UNIX:
+        return UV_NAMED_PIPE;
+      case AF_INET:
+      case AF_INET6:
+        return UV_TCP;
+      }
   }
 
+  if (type == SOCK_DGRAM &&
+      (ss.ss_family == AF_INET || ss.ss_family == AF_INET6))
+    return UV_UDP;
+
   return UV_UNKNOWN_HANDLE;
 }
 
@@ -879,11 +949,11 @@ static void uv__read(uv_stream_t* stream) {
 
     assert(buf.len > 0);
     assert(buf.base);
-    assert(stream->fd >= 0);
+    assert(uv__stream_fd(stream) >= 0);
 
     if (stream->read_cb) {
       do {
-        nread = read(stream->fd, buf.base, buf.len);
+        nread = read(uv__stream_fd(stream), buf.base, buf.len);
       }
       while (nread < 0 && errno == EINTR);
     } else {
@@ -899,54 +969,44 @@ static void uv__read(uv_stream_t* stream) {
       msg.msg_control = (void *) cmsg_space;
 
       do {
-        nread = recvmsg(stream->fd, &msg, 0);
+        nread = recvmsg(uv__stream_fd(stream), &msg, 0);
       }
       while (nread < 0 && errno == EINTR);
     }
 
+#define INVOKE_READ_CB(stream, status, buf, type)                             \
+    do {                                                                      \
+      if ((stream)->read_cb != NULL)                                          \
+        (stream)->read_cb((stream), (status), (buf));                         \
+      else                                                                    \
+        (stream)->read2_cb((uv_pipe_t*) (stream), (status), (buf), (type));   \
+    }                                                                         \
+    while (0)
 
     if (nread < 0) {
       /* Error */
       if (errno == EAGAIN || errno == EWOULDBLOCK) {
         /* Wait for the next one. */
         if (stream->flags & UV_STREAM_READING) {
-          uv__io_start(stream->loop, &stream->read_watcher);
+          uv__io_start(stream->loop, &stream->io_watcher, UV__POLLIN);
         }
         uv__set_sys_error(stream->loop, EAGAIN);
-
-        if (stream->read_cb) {
-          stream->read_cb(stream, 0, buf);
-        } else {
-          stream->read2_cb((uv_pipe_t*)stream, 0, buf, UV_UNKNOWN_HANDLE);
-        }
-
-        return;
+        INVOKE_READ_CB(stream, 0, buf, UV_UNKNOWN_HANDLE);
       } else {
         /* Error. User should call uv_close(). */
         uv__set_sys_error(stream->loop, errno);
-
-        if (stream->read_cb) {
-          stream->read_cb(stream, -1, buf);
-        } else {
-          stream->read2_cb((uv_pipe_t*)stream, -1, buf, UV_UNKNOWN_HANDLE);
-        }
-
-        assert(!uv__io_active(&stream->read_watcher));
-        return;
+        INVOKE_READ_CB(stream, -1, buf, UV_UNKNOWN_HANDLE);
+        assert(!uv__io_active(&stream->io_watcher, UV__POLLIN) &&
+               "stream->read_cb(status=-1) did not call uv_close()");
       }
-
+      return;
     } else if (nread == 0) {
       /* EOF */
-      uv__set_artificial_error(stream->loop, UV_EOF);
-      uv__io_stop(stream->loop, &stream->read_watcher);
-      if (!uv__io_active(&stream->write_watcher))
+      uv__io_stop(stream->loop, &stream->io_watcher, UV__POLLIN);
+      if (!uv__io_active(&stream->io_watcher, UV__POLLOUT))
         uv__handle_stop(stream);
-
-      if (stream->read_cb) {
-        stream->read_cb(stream, -1, buf);
-      } else {
-        stream->read2_cb((uv_pipe_t*)stream, -1, buf, UV_UNKNOWN_HANDLE);
-      }
+      uv__set_artificial_error(stream->loop, UV_EOF);
+      INVOKE_READ_CB(stream, -1, buf, UV_UNKNOWN_HANDLE);
       return;
     } else {
       /* Successful read */
@@ -1007,7 +1067,7 @@ static void uv__read(uv_stream_t* stream) {
 int uv_shutdown(uv_shutdown_t* req, uv_stream_t* stream, uv_shutdown_cb cb) {
   assert((stream->type == UV_TCP || stream->type == UV_NAMED_PIPE) &&
          "uv_shutdown (unix) only supports uv_handle_t right now");
-  assert(stream->fd >= 0);
+  assert(uv__stream_fd(stream) >= 0);
 
   if (!(stream->flags & UV_STREAM_WRITABLE) ||
       stream->flags & UV_STREAM_SHUT ||
@@ -1024,36 +1084,38 @@ int uv_shutdown(uv_shutdown_t* req, uv_stream_t* stream, uv_shutdown_cb cb) {
   stream->shutdown_req = req;
   stream->flags |= UV_STREAM_SHUTTING;
 
-  uv__io_start(stream->loop, &stream->write_watcher);
+  uv__io_start(stream->loop, &stream->io_watcher, UV__POLLOUT);
 
   return 0;
 }
 
 
-static void uv__stream_io(uv_loop_t* loop, uv__io_t* w, int events) {
+static void uv__stream_io(uv_loop_t* loop, uv__io_t* w, unsigned int events) {
   uv_stream_t* stream;
 
-  /* either UV__IO_READ or UV__IO_WRITE but not both */
-  assert(!!(events & UV__IO_READ) ^ !!(events & UV__IO_WRITE));
-
-  if (events & UV__IO_READ)
-    stream = container_of(w, uv_stream_t, read_watcher);
-  else
-    stream = container_of(w, uv_stream_t, write_watcher);
+  stream = container_of(w, uv_stream_t, io_watcher);
 
   assert(stream->type == UV_TCP ||
          stream->type == UV_NAMED_PIPE ||
          stream->type == UV_TTY);
   assert(!(stream->flags & UV_CLOSING));
 
-  if (stream->connect_req)
+  if (stream->connect_req) {
     uv__stream_connect(stream);
-  else if (events & UV__IO_READ) {
-    assert(stream->fd >= 0);
-    uv__read(stream);
+    return;
   }
-  else {
-    assert(stream->fd >= 0);
+
+  if (events & (UV__POLLIN | UV__POLLERR | UV__POLLHUP)) {
+    assert(uv__stream_fd(stream) >= 0);
+
+    uv__read(stream);
+
+    if (uv__stream_fd(stream) == -1)
+      return; /* read_cb closed stream. */
+  }
+
+  if (events & UV__POLLOUT) {
+    assert(uv__stream_fd(stream) >= 0);
     uv__write(stream);
     uv__write_callbacks(stream);
   }
@@ -1082,8 +1144,12 @@ static void uv__stream_connect(uv_stream_t* stream) {
     stream->delayed_error = 0;
   } else {
     /* Normal situation: we need to get the socket error from the kernel. */
-    assert(stream->fd >= 0);
-    getsockopt(stream->fd, SOL_SOCKET, SO_ERROR, &error, &errorsize);
+    assert(uv__stream_fd(stream) >= 0);
+    getsockopt(uv__stream_fd(stream),
+               SOL_SOCKET,
+               SO_ERROR,
+               &error,
+               &errorsize);
   }
 
   if (error == EINPROGRESS)
@@ -1108,21 +1174,26 @@ int uv_write2(uv_write_t* req,
   int empty_queue;
 
   assert(bufcnt > 0);
+  assert((stream->type == UV_TCP ||
+          stream->type == UV_NAMED_PIPE ||
+          stream->type == UV_TTY) &&
+         "uv_write (unix) does not yet support other types of streams");
 
-  assert((stream->type == UV_TCP || stream->type == UV_NAMED_PIPE ||
-      stream->type == UV_TTY) &&
-      "uv_write (unix) does not yet support other types of streams");
-
-  if (stream->fd < 0) {
-    uv__set_sys_error(stream->loop, EBADF);
-    return -1;
-  }
+  if (uv__stream_fd(stream) < 0)
+    return uv__set_artificial_error(stream->loop, UV_EBADF);
 
   if (send_handle) {
-    if (stream->type != UV_NAMED_PIPE || !((uv_pipe_t*)stream)->ipc) {
-      uv__set_sys_error(stream->loop, EOPNOTSUPP);
-      return -1;
-    }
+    if (stream->type != UV_NAMED_PIPE || !((uv_pipe_t*)stream)->ipc)
+      return uv__set_artificial_error(stream->loop, UV_EINVAL);
+
+    /* XXX We abuse uv_write2() to send over UDP handles to child processes.
+     * Don't call uv__stream_fd() on those handles, it's a macro that on OS X
+     * evaluates to a function that operates on a uv_stream_t with a couple of
+     * OS X specific fields. On other Unices it does (handle)->io_watcher.fd,
+     * which works but only by accident.
+     */
+    if (uv__handle_fd((uv_handle_t*) send_handle) < 0)
+      return uv__set_artificial_error(stream->loop, UV_EBADF);
   }
 
   empty_queue = (stream->write_queue_size == 0);
@@ -1165,7 +1236,7 @@ int uv_write2(uv_write_t* req,
      * sufficiently flushed in uv__write.
      */
     assert(!(stream->flags & UV_STREAM_BLOCKING));
-    uv__io_start(stream->loop, &stream->write_watcher);
+    uv__io_start(stream->loop, &stream->io_watcher, UV__POLLOUT);
   }
 
   return 0;
@@ -1181,33 +1252,39 @@ int uv_write(uv_write_t* req, uv_stream_t* stream, uv_buf_t bufs[], int bufcnt,
 }
 
 
-int uv__read_start_common(uv_stream_t* stream, uv_alloc_cb alloc_cb,
-    uv_read_cb read_cb, uv_read2_cb read2_cb) {
+static int uv__read_start_common(uv_stream_t* stream,
+                                 uv_alloc_cb alloc_cb,
+                                 uv_read_cb read_cb,
+                                 uv_read2_cb read2_cb) {
   assert(stream->type == UV_TCP || stream->type == UV_NAMED_PIPE ||
       stream->type == UV_TTY);
 
-  if (stream->flags & UV_CLOSING) {
-    uv__set_sys_error(stream->loop, EINVAL);
-    return -1;
-  }
+  if (stream->flags & UV_CLOSING)
+    return uv__set_sys_error(stream->loop, EINVAL);
 
   /* The UV_STREAM_READING flag is irrelevant of the state of the tcp - it just
    * expresses the desired state of the user.
    */
   stream->flags |= UV_STREAM_READING;
 
+#if defined(__APPLE__)
+  /* Notify select() thread about state change */
+  if (stream->select != NULL)
+    uv__stream_osx_interrupt_select(stream);
+#endif /* defined(__APPLE__) */
+
   /* TODO: try to do the read inline? */
   /* TODO: keep track of tcp state. If we've gotten a EOF then we should
    * not start the IO watcher.
    */
-  assert(stream->fd >= 0);
+  assert(uv__stream_fd(stream) >= 0);
   assert(alloc_cb);
 
   stream->read_cb = read_cb;
   stream->read2_cb = read2_cb;
   stream->alloc_cb = alloc_cb;
 
-  uv__io_start(stream->loop, &stream->read_watcher);
+  uv__io_start(stream->loop, &stream->io_watcher, UV__POLLIN);
   uv__handle_start(stream);
 
   return 0;
@@ -1227,9 +1304,16 @@ int uv_read2_start(uv_stream_t* stream, uv_alloc_cb alloc_cb,
 
 
 int uv_read_stop(uv_stream_t* stream) {
-  uv__io_stop(stream->loop, &stream->read_watcher);
+  uv__io_stop(stream->loop, &stream->io_watcher, UV__POLLIN);
   uv__handle_stop(stream);
   stream->flags &= ~UV_STREAM_READING;
+
+#if defined(__APPLE__)
+  /* Notify select() thread about state change */
+  if (stream->select != NULL)
+    uv__stream_osx_interrupt_select(stream);
+#endif /* defined(__APPLE__) */
+
   stream->read_cb = NULL;
   stream->read2_cb = NULL;
   stream->alloc_cb = NULL;
@@ -1247,6 +1331,23 @@ int uv_is_writable(const uv_stream_t* stream) {
 }
 
 
+#if defined(__APPLE__)
+int uv___stream_fd(uv_stream_t* handle) {
+  uv__stream_select_t* s;
+
+  assert(handle->type == UV_TCP ||
+         handle->type == UV_TTY ||
+         handle->type == UV_NAMED_PIPE);
+
+  s = handle->select;
+  if (s != NULL)
+    return s->fd;
+
+  return handle->io_watcher.fd;
+}
+#endif /* defined(__APPLE__) */
+
+
 void uv__stream_close(uv_stream_t* handle) {
 #if defined(__APPLE__)
   /* Terminate select loop first */
@@ -1256,10 +1357,12 @@ void uv__stream_close(uv_stream_t* handle) {
     s = handle->select;
 
     uv_sem_post(&s->sem);
+    uv__stream_osx_interrupt_select(handle);
     uv_thread_join(&s->thread);
     uv_sem_destroy(&s->sem);
     uv_mutex_destroy(&s->mutex);
     close(s->fake_fd);
+    close(s->int_fd);
     uv_close((uv_handle_t*) &s->async, uv__stream_osx_cb_close);
 
     handle->select = NULL;
@@ -1267,16 +1370,15 @@ void uv__stream_close(uv_stream_t* handle) {
 #endif /* defined(__APPLE__) */
 
   uv_read_stop(handle);
-  uv__io_stop(handle->loop, &handle->write_watcher);
+  uv__io_close(handle->loop, &handle->io_watcher);
 
-  close(handle->fd);
-  handle->fd = -1;
+  close(handle->io_watcher.fd);
+  handle->io_watcher.fd = -1;
 
   if (handle->accepted_fd >= 0) {
     close(handle->accepted_fd);
     handle->accepted_fd = -1;
   }
 
-  assert(!uv__io_active(&handle->read_watcher));
-  assert(!uv__io_active(&handle->write_watcher));
+  assert(!uv__io_active(&handle->io_watcher, UV__POLLIN | UV__POLLOUT));
 }
