@@ -56,14 +56,24 @@ int uv_udp_getsockname(uv_udp_t* handle, struct sockaddr* name,
 }
 
 
-static int uv_udp_set_socket(uv_loop_t* loop, uv_udp_t* handle,
-    SOCKET socket) {
+static int uv_udp_set_socket(uv_loop_t* loop, uv_udp_t* handle, SOCKET socket,
+    int family) {
   DWORD yes = 1;
   WSAPROTOCOL_INFOW info;
   int opt_len;
 
   assert(handle->socket == INVALID_SOCKET);
 
+  /* Set SO_REUSEADDR on the socket. */
+  if (setsockopt(socket,
+                 SOL_SOCKET,
+                 SO_REUSEADDR,
+                 (char*) &yes,
+                 sizeof yes) == SOCKET_ERROR) {
+    uv__set_sys_error(loop, WSAGetLastError());
+    return -1;
+  }
+
   /* Set the socket to nonblocking mode */
   if (ioctlsocket(socket, FIONBIO, &yes) == SOCKET_ERROR) {
     uv__set_sys_error(loop, WSAGetLastError());
@@ -93,7 +103,7 @@ static int uv_udp_set_socket(uv_loop_t* loop, uv_udp_t* handle,
     /* if the user is using the default UDP driver (AFD) and has no other */
     /* LSPs stacked on top. Here we check whether that is the case. */
     opt_len = (int) sizeof info;
-    if (!getsockopt(socket,
+    if (getsockopt(socket,
                    SOL_SOCKET,
                    SO_PROTOCOL_INFOW,
                    (char*) &info,
@@ -118,6 +128,12 @@ static int uv_udp_set_socket(uv_loop_t* loop, uv_udp_t* handle,
 
   handle->socket = socket;
 
+  if (family == AF_INET6) {
+    handle->flags |= UV_HANDLE_IPV6;
+  } else {
+    assert(!(handle->flags & UV_HANDLE_IPV6));
+  }
+
   return 0;
 }
 
@@ -161,33 +177,36 @@ void uv_udp_endgame(uv_loop_t* loop, uv_udp_t* handle) {
 
 
 static int uv__bind(uv_udp_t* handle,
-                    int domain,
+                    int family,
                     struct sockaddr* addr,
                     int addrsize,
                     unsigned int flags) {
   int r;
   DWORD no = 0, yes = 1;
 
-  if ((flags & UV_UDP_IPV6ONLY) && domain != AF_INET6) {
+  if ((flags & UV_UDP_IPV6ONLY) && family != AF_INET6) {
     /* UV_UDP_IPV6ONLY is supported only for IPV6 sockets */
     uv__set_artificial_error(handle->loop, UV_EINVAL);
     return -1;
   }
 
   if (handle->socket == INVALID_SOCKET) {
-    SOCKET sock = socket(domain, SOCK_DGRAM, 0);
+    SOCKET sock = socket(family, SOCK_DGRAM, 0);
     if (sock == INVALID_SOCKET) {
       uv__set_sys_error(handle->loop, WSAGetLastError());
       return -1;
     }
 
-    if (uv_udp_set_socket(handle->loop, handle, sock) == -1) {
+    if (uv_udp_set_socket(handle->loop, handle, sock, family) < 0) {
       closesocket(sock);
       return -1;
     }
+
+    if (family == AF_INET6)
+      handle->flags |= UV_HANDLE_IPV6;
   }
 
-  if (domain == AF_INET6 && !(flags & UV_UDP_IPV6ONLY)) {
+  if (family == AF_INET6 && !(flags & UV_UDP_IPV6ONLY)) {
     /* On windows IPV6ONLY is on by default. */
     /* If the user doesn't specify it libuv turns it off. */
 
@@ -201,16 +220,6 @@ static int uv__bind(uv_udp_t* handle,
                sizeof no);
   }
 
-  r = setsockopt(handle->socket,
-                 SOL_SOCKET,
-                 SO_REUSEADDR,
-                 (char*) &yes,
-                 sizeof yes);
-  if (r == SOCKET_ERROR) {
-    uv__set_sys_error(handle->loop, WSAGetLastError());
-    return -1;
-  }
-
   r = bind(handle->socket, addr, addrsize);
   if (r == SOCKET_ERROR) {
     uv__set_sys_error(handle->loop, WSAGetLastError());
@@ -235,17 +244,11 @@ int uv__udp_bind(uv_udp_t* handle, struct sockaddr_in addr,
 
 int uv__udp_bind6(uv_udp_t* handle, struct sockaddr_in6 addr,
     unsigned int flags) {
-  if (uv_allow_ipv6) {
-    handle->flags |= UV_HANDLE_IPV6;
-    return uv__bind(handle,
-                    AF_INET6,
-                    (struct sockaddr*) &addr,
-                    sizeof(struct sockaddr_in6),
-                    flags);
-  } else {
-    uv__set_sys_error(handle->loop, WSAEAFNOSUPPORT);
-    return -1;
-  }
+  return uv__bind(handle,
+                  AF_INET6,
+                  (struct sockaddr*) &addr,
+                  sizeof(struct sockaddr_in6),
+                  flags);
 }
 
 
@@ -338,7 +341,7 @@ static void uv_udp_queue_recv(uv_loop_t* loop, uv_udp_t* handle) {
 }
 
 
-int uv_udp_recv_start(uv_udp_t* handle, uv_alloc_cb alloc_cb,
+int uv__udp_recv_start(uv_udp_t* handle, uv_alloc_cb alloc_cb,
     uv_udp_recv_cb recv_cb) {
   uv_loop_t* loop = handle->loop;
 
@@ -368,7 +371,7 @@ int uv_udp_recv_start(uv_udp_t* handle, uv_alloc_cb alloc_cb,
 }
 
 
-int uv_udp_recv_stop(uv_udp_t* handle) {
+int uv__udp_recv_stop(uv_udp_t* handle) {
   if (handle->flags & UV_HANDLE_READING) {
     handle->flags &= ~UV_HANDLE_READING;
     handle->loop->active_udp_streams--;
@@ -379,7 +382,7 @@ int uv_udp_recv_stop(uv_udp_t* handle) {
 }
 
 
-static int uv__udp_send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
+static int uv__send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
     int bufcnt, struct sockaddr* addr, int addr_len, uv_udp_send_cb cb) {
   uv_loop_t* loop = handle->loop;
   DWORD result, bytes;
@@ -421,7 +424,7 @@ static int uv__udp_send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
 }
 
 
-int uv_udp_send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
+int uv__udp_send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
     int bufcnt, struct sockaddr_in addr, uv_udp_send_cb cb) {
 
   if (!(handle->flags & UV_HANDLE_BOUND) &&
@@ -429,17 +432,17 @@ int uv_udp_send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
     return -1;
   }
 
-  return uv__udp_send(req,
-                      handle,
-                      bufs,
-                      bufcnt,
-                      (struct sockaddr*) &addr,
-                      sizeof addr,
-                      cb);
+  return uv__send(req,
+                  handle,
+                  bufs,
+                  bufcnt,
+                  (struct sockaddr*) &addr,
+                  sizeof addr,
+                  cb);
 }
 
 
-int uv_udp_send6(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
+int uv__udp_send6(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
     int bufcnt, struct sockaddr_in6 addr, uv_udp_send_cb cb) {
 
   if (!(handle->flags & UV_HANDLE_BOUND) &&
@@ -447,13 +450,13 @@ int uv_udp_send6(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
     return -1;
   }
 
-  return uv__udp_send(req,
-                      handle,
-                      bufs,
-                      bufcnt,
-                      (struct sockaddr*) &addr,
-                      sizeof addr,
-                      cb);
+  return uv__send(req,
+                  handle,
+                  bufs,
+                  bufcnt,
+                  (struct sockaddr*) &addr,
+                  sizeof addr,
+                  cb);
 }
 
 
@@ -615,8 +618,7 @@ int uv_udp_set_membership(uv_udp_t* handle, const char* multicast_addr,
       optname = IP_DROP_MEMBERSHIP;
       break;
     default:
-      uv__set_artificial_error(handle->loop, UV_EFAULT);
-      return -1;
+      return uv__set_artificial_error(handle->loop, UV_EINVAL);
   }
 
   if (setsockopt(handle->socket,
@@ -642,10 +644,10 @@ int uv_udp_set_broadcast(uv_udp_t* handle, int value) {
   }
 
   if (setsockopt(handle->socket,
-                  SOL_SOCKET,
-                  SO_BROADCAST,
-                  (char*) &optval,
-                  sizeof optval)) {
+                 SOL_SOCKET,
+                 SO_BROADCAST,
+                 (char*) &optval,
+                 sizeof optval)) {
     uv__set_sys_error(handle->loop, WSAGetLastError());
     return -1;
   }
@@ -654,20 +656,25 @@ int uv_udp_set_broadcast(uv_udp_t* handle, int value) {
 
 
 int uv_udp_open(uv_udp_t* handle, uv_os_sock_t sock) {
-  int r;
+  WSAPROTOCOL_INFOW protocol_info;
+  int opt_len;
   DWORD yes = 1;
 
-  if (uv_udp_set_socket(handle->loop, handle, sock) == -1) {
+  /* Detect the address family of the socket. */
+  opt_len = (int) sizeof protocol_info;
+  if (getsockopt(sock,
+                 SOL_SOCKET,
+                 SO_PROTOCOL_INFOW,
+                 (char*) &protocol_info,
+                 &opt_len) == SOCKET_ERROR) {
+    uv__set_sys_error(handle->loop, GetLastError());
     return -1;
   }
 
-  r = setsockopt(handle->socket,
-                 SOL_SOCKET,
-                 SO_REUSEADDR,
-                 (char*) &yes,
-                 sizeof yes);
-  if (r == SOCKET_ERROR) {
-    uv__set_sys_error(handle->loop, WSAGetLastError());
+  if (uv_udp_set_socket(handle->loop,
+                        handle,
+                        sock,
+                        protocol_info.iAddressFamily) < 0) {
     return -1;
   }
 
