@@ -38,7 +38,7 @@
 typedef struct {
   char *task_name;
   char *process_name;
-  int (*main)();
+  int (*main)(void);
   int is_helper;
   int show_output;
 } task_entry_t, bench_entry_t;
@@ -51,11 +51,11 @@ typedef struct {
   task_entry_t TASKS[] = {
 
 #define TASK_LIST_END                               \
-    { 0, 0, 0, 0 }                                  \
+    { 0, 0, 0, 0, 0 }                               \
   };
 
 #define TEST_DECLARE(name)                          \
-  int run_test_##name();
+  int run_test_##name(void);
 
 #define TEST_ENTRY(name)                            \
     { #name, #name, &run_test_##name, 0, 0 },
@@ -64,13 +64,13 @@ typedef struct {
     { #name, #name, &run_test_##name, 0, 1 },
 
 #define BENCHMARK_DECLARE(name)                     \
-  int run_benchmark_##name();
+  int run_benchmark_##name(void);
 
 #define BENCHMARK_ENTRY(name)                       \
     { #name, #name, &run_benchmark_##name, 0, 0 },
 
 #define HELPER_DECLARE(name)                        \
-  int run_helper_##name();
+  int run_helper_##name(void);
 
 #define HELPER_ENTRY(task_name, name)               \
     { #task_name, #name, &run_helper_##name, 1, 0 },
@@ -102,7 +102,10 @@ int run_tests(int timeout, int benchmark_output);
 /*
  * Run a single test. Starts up any helpers.
  */
-int run_test(const char* test, int timeout, int benchmark_output);
+int run_test(const char* test,
+             int timeout,
+             int benchmark_output,
+             int test_count);
 
 /*
  * Run a test part, i.e. the test or one of its helpers.
@@ -123,7 +126,7 @@ void print_tests(FILE* stream);
  */
 
 /* Do platform-specific initialization. */
-void platform_init();
+void platform_init(int argc, char** argv);
 
 /* Invoke "argv[0] test-name [test-part]". Store process info in *p. */
 /* Make sure that all stdio output of the processes is buffered up. */
@@ -154,6 +157,9 @@ int process_reap(process_info_t *p);
 void process_cleanup(process_info_t *p);
 
 /* Move the console cursor one line up and back to the first column. */
-void rewind_cursor();
+void rewind_cursor(void);
+
+/* trigger output as tap */
+extern int tap_output;
 
 #endif /* RUNNER_H_ */
