@@ -76,7 +76,9 @@ static void poll_cb(uv_fs_poll_t* handle,
                     int status,
                     const uv_statbuf_t* prev,
                     const uv_statbuf_t* curr) {
-  const static uv_statbuf_t zero_statbuf;
+  uv_statbuf_t zero_statbuf;
+
+  memset(&zero_statbuf, 0, sizeof(zero_statbuf));
 
   ASSERT(handle == &poll_handle);
   ASSERT(uv_is_active((uv_handle_t*)handle));
@@ -135,11 +137,12 @@ TEST_IMPL(fs_poll) {
   ASSERT(0 == uv_timer_init(loop, &timer_handle));
   ASSERT(0 == uv_fs_poll_init(loop, &poll_handle));
   ASSERT(0 == uv_fs_poll_start(&poll_handle, poll_cb, FIXTURE, 100));
-  ASSERT(0 == uv_run(loop));
+  ASSERT(0 == uv_run(loop, UV_RUN_DEFAULT));
 
   ASSERT(poll_cb_called == 5);
   ASSERT(timer_cb_called == 2);
   ASSERT(close_cb_called == 1);
 
+  MAKE_VALGRIND_HAPPY();
   return 0;
 }
