@@ -22,6 +22,16 @@
 TEST_DECLARE   (platform_output)
 TEST_DECLARE   (callback_order)
 TEST_DECLARE   (run_once)
+TEST_DECLARE   (run_nowait)
+TEST_DECLARE   (loop_stop)
+TEST_DECLARE   (barrier_1)
+TEST_DECLARE   (barrier_2)
+TEST_DECLARE   (barrier_3)
+TEST_DECLARE   (condvar_1)
+TEST_DECLARE   (condvar_2)
+TEST_DECLARE   (condvar_3)
+TEST_DECLARE   (condvar_4)
+TEST_DECLARE   (condvar_5)
 TEST_DECLARE   (semaphore_1)
 TEST_DECLARE   (semaphore_2)
 TEST_DECLARE   (semaphore_3)
@@ -55,9 +65,9 @@ TEST_DECLARE   (tcp_connect_timeout)
 TEST_DECLARE   (tcp_close_while_connecting)
 TEST_DECLARE   (tcp_close)
 TEST_DECLARE   (tcp_flags)
-TEST_DECLARE   (tcp_write_error)
 TEST_DECLARE   (tcp_write_to_half_open_connection)
 TEST_DECLARE   (tcp_unexpected_read)
+TEST_DECLARE   (tcp_read_stop)
 TEST_DECLARE   (tcp_bind6_error_addrinuse)
 TEST_DECLARE   (tcp_bind6_error_addrnotavail)
 TEST_DECLARE   (tcp_bind6_error_fault)
@@ -85,8 +95,12 @@ TEST_DECLARE   (shutdown_eof)
 TEST_DECLARE   (callback_stack)
 TEST_DECLARE   (error_message)
 TEST_DECLARE   (timer)
+TEST_DECLARE   (timer_init)
 TEST_DECLARE   (timer_again)
 TEST_DECLARE   (timer_start_twice)
+TEST_DECLARE   (timer_order)
+TEST_DECLARE   (timer_huge_timeout)
+TEST_DECLARE   (timer_huge_repeat)
 TEST_DECLARE   (idle_starvation)
 TEST_DECLARE   (loop_handles)
 TEST_DECLARE   (get_loadavg)
@@ -103,6 +117,7 @@ TEST_DECLARE   (fs_event_ref)
 TEST_DECLARE   (fs_poll_ref)
 TEST_DECLARE   (tcp_ref)
 TEST_DECLARE   (tcp_ref2)
+TEST_DECLARE   (tcp_ref2b)
 TEST_DECLARE   (tcp_ref3)
 TEST_DECLARE   (tcp_ref4)
 TEST_DECLARE   (udp_ref)
@@ -114,6 +129,7 @@ TEST_DECLARE   (pipe_ref3)
 TEST_DECLARE   (pipe_ref4)
 TEST_DECLARE   (process_ref)
 TEST_DECLARE   (active)
+TEST_DECLARE   (embed)
 TEST_DECLARE   (async)
 TEST_DECLARE   (get_currentexe)
 TEST_DECLARE   (process_title)
@@ -136,9 +152,11 @@ TEST_DECLARE   (spawn_and_kill)
 TEST_DECLARE   (spawn_detached)
 TEST_DECLARE   (spawn_and_kill_with_std)
 TEST_DECLARE   (spawn_and_ping)
+TEST_DECLARE   (spawn_preserve_env)
 TEST_DECLARE   (spawn_setuid_fails)
 TEST_DECLARE   (spawn_setgid_fails)
 TEST_DECLARE   (spawn_stdout_to_file)
+TEST_DECLARE   (spawn_auto_unref)
 TEST_DECLARE   (fs_poll)
 TEST_DECLARE   (kill)
 TEST_DECLARE   (fs_file_noent)
@@ -164,6 +182,7 @@ TEST_DECLARE   (fs_event_watch_dir)
 TEST_DECLARE   (fs_event_watch_file)
 TEST_DECLARE   (fs_event_watch_file_twice)
 TEST_DECLARE   (fs_event_watch_file_current_dir)
+TEST_DECLARE   (fs_event_no_callback_after_close)
 TEST_DECLARE   (fs_event_no_callback_on_close)
 TEST_DECLARE   (fs_event_immediate_close)
 TEST_DECLARE   (fs_event_close_with_pending_event)
@@ -173,7 +192,12 @@ TEST_DECLARE   (fs_readdir_file)
 TEST_DECLARE   (fs_open_dir)
 TEST_DECLARE   (fs_rename_to_existing_file)
 TEST_DECLARE   (threadpool_queue_work_simple)
+TEST_DECLARE   (threadpool_queue_work_einval)
 TEST_DECLARE   (threadpool_multiple_event_loops)
+TEST_DECLARE   (threadpool_cancel_getaddrinfo)
+TEST_DECLARE   (threadpool_cancel_work)
+TEST_DECLARE   (threadpool_cancel_fs)
+TEST_DECLARE   (threadpool_cancel_single)
 TEST_DECLARE   (thread_mutex)
 TEST_DECLARE   (thread_rwlock)
 TEST_DECLARE   (thread_create)
@@ -183,8 +207,6 @@ TEST_DECLARE   (dlerror)
 TEST_DECLARE   (poll_duplex)
 TEST_DECLARE   (poll_unidirectional)
 TEST_DECLARE   (poll_close)
-TEST_DECLARE   (we_get_signal)
-TEST_DECLARE   (we_get_signals)
 #ifdef _WIN32
 TEST_DECLARE   (spawn_detect_pipe_name_collisions_on_windows)
 TEST_DECLARE   (argument_escaping)
@@ -194,6 +216,9 @@ TEST_DECLARE   (listen_no_simultaneous_accepts)
 TEST_DECLARE   (fs_stat_root)
 #else
 TEST_DECLARE   (spawn_setuid_setgid)
+TEST_DECLARE   (we_get_signal)
+TEST_DECLARE   (we_get_signals)
+TEST_DECLARE   (signal_multiple_loops)
 #endif
 HELPER_DECLARE (tcp4_echo_server)
 HELPER_DECLARE (tcp6_echo_server)
@@ -208,6 +233,16 @@ TASK_LIST_START
   TEST_ENTRY  (callback_order)
 #endif
   TEST_ENTRY  (run_once)
+  TEST_ENTRY  (run_nowait)
+  TEST_ENTRY  (loop_stop)
+  TEST_ENTRY  (barrier_1)
+  TEST_ENTRY  (barrier_2)
+  TEST_ENTRY  (barrier_3)
+  TEST_ENTRY  (condvar_1)
+  TEST_ENTRY  (condvar_2)
+  TEST_ENTRY  (condvar_3)
+  TEST_ENTRY  (condvar_4)
+  TEST_ENTRY  (condvar_5)
   TEST_ENTRY  (semaphore_1)
   TEST_ENTRY  (semaphore_2)
   TEST_ENTRY  (semaphore_3)
@@ -259,10 +294,12 @@ TASK_LIST_START
   TEST_ENTRY  (tcp_close_while_connecting)
   TEST_ENTRY  (tcp_close)
   TEST_ENTRY  (tcp_flags)
-  TEST_ENTRY  (tcp_write_error)
   TEST_ENTRY  (tcp_write_to_half_open_connection)
   TEST_ENTRY  (tcp_unexpected_read)
 
+  TEST_ENTRY  (tcp_read_stop)
+  TEST_HELPER (tcp_read_stop, tcp4_echo_server)
+
   TEST_ENTRY  (tcp_bind6_error_addrinuse)
   TEST_ENTRY  (tcp_bind6_error_addrnotavail)
   TEST_ENTRY  (tcp_bind6_error_fault)
@@ -302,8 +339,12 @@ TASK_LIST_START
   TEST_ENTRY  (error_message)
 
   TEST_ENTRY  (timer)
+  TEST_ENTRY  (timer_init)
   TEST_ENTRY  (timer_again)
   TEST_ENTRY  (timer_start_twice)
+  TEST_ENTRY  (timer_order)
+  TEST_ENTRY  (timer_huge_timeout)
+  TEST_ENTRY  (timer_huge_repeat)
 
   TEST_ENTRY  (idle_starvation)
 
@@ -319,6 +360,7 @@ TASK_LIST_START
   TEST_ENTRY  (fs_event_ref)
   TEST_ENTRY  (tcp_ref)
   TEST_ENTRY  (tcp_ref2)
+  TEST_ENTRY  (tcp_ref2b)
   TEST_ENTRY  (tcp_ref3)
   TEST_HELPER (tcp_ref3, tcp4_echo_server)
   TEST_ENTRY  (tcp_ref4)
@@ -340,6 +382,8 @@ TASK_LIST_START
 
   TEST_ENTRY  (active)
 
+  TEST_ENTRY  (embed)
+
   TEST_ENTRY  (async)
 
   TEST_ENTRY  (get_currentexe)
@@ -374,15 +418,14 @@ TASK_LIST_START
   TEST_ENTRY  (spawn_detached)
   TEST_ENTRY  (spawn_and_kill_with_std)
   TEST_ENTRY  (spawn_and_ping)
+  TEST_ENTRY  (spawn_preserve_env)
   TEST_ENTRY  (spawn_setuid_fails)
   TEST_ENTRY  (spawn_setgid_fails)
   TEST_ENTRY  (spawn_stdout_to_file)
+  TEST_ENTRY  (spawn_auto_unref)
   TEST_ENTRY  (fs_poll)
   TEST_ENTRY  (kill)
 
-  TEST_ENTRY  (we_get_signal)
-  TEST_ENTRY  (we_get_signals)
-
 #ifdef _WIN32
   TEST_ENTRY  (spawn_detect_pipe_name_collisions_on_windows)
   TEST_ENTRY  (argument_escaping)
@@ -392,6 +435,9 @@ TASK_LIST_START
   TEST_ENTRY  (fs_stat_root)
 #else
   TEST_ENTRY  (spawn_setuid_setgid)
+  TEST_ENTRY  (we_get_signal)
+  TEST_ENTRY  (we_get_signals)
+  TEST_ENTRY  (signal_multiple_loops)
 #endif
 
   TEST_ENTRY  (fs_file_noent)
@@ -416,6 +462,7 @@ TASK_LIST_START
   TEST_ENTRY  (fs_event_watch_file)
   TEST_ENTRY  (fs_event_watch_file_twice)
   TEST_ENTRY  (fs_event_watch_file_current_dir)
+  TEST_ENTRY  (fs_event_no_callback_after_close)
   TEST_ENTRY  (fs_event_no_callback_on_close)
   TEST_ENTRY  (fs_event_immediate_close)
   TEST_ENTRY  (fs_event_close_with_pending_event)
@@ -425,7 +472,12 @@ TASK_LIST_START
   TEST_ENTRY  (fs_open_dir)
   TEST_ENTRY  (fs_rename_to_existing_file)
   TEST_ENTRY  (threadpool_queue_work_simple)
+  TEST_ENTRY  (threadpool_queue_work_einval)
   TEST_ENTRY  (threadpool_multiple_event_loops)
+  TEST_ENTRY  (threadpool_cancel_getaddrinfo)
+  TEST_ENTRY  (threadpool_cancel_work)
+  TEST_ENTRY  (threadpool_cancel_fs)
+  TEST_ENTRY  (threadpool_cancel_single)
   TEST_ENTRY  (thread_mutex)
   TEST_ENTRY  (thread_rwlock)
   TEST_ENTRY  (thread_create)
